<?php

namespace Drupal\lametric\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Drupal\lametric\Service\LaMetricNotificationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class LaMetricForm.
 *
 * @package Drupal\lametric\Form
 */
class LaMetricForm extends FormBase {

  /**
   * The LaMetric notification.
   *
   * @var \Drupal\lametric\Service\LaMetricNotificationInterface
   */
  protected $notification;

  /**
   * LaMetricForm constructor.
   *
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The string translation service.
   * @param \Drupal\lametric\Service\LaMetricNotificationInterface $notification
   *   The LaMetric notification.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(
    TranslationInterface $translation,
    LaMetricNotificationInterface $notification,
    MessengerInterface $messenger
  ) {
    $this->setStringTranslation($translation);
    $this->notification = $notification;
    $this->setMessenger($messenger);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('string_translation'),
      $container->get('lametric.notification'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'lametric_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $options = [
      'attributes' => [
        'target' => '_blank',
      ],
    ];

    $url = Url::fromUri('https://lametric-documentation.readthedocs.io/en/latest/_images/lametric_time_ip_address_android.png', $options);

    $form['ip'] = [
      '#type' => 'textfield',
      '#title' => $this->t('IP address'),
      '#description' => $this->t('See @url where you can found this data.', [
        '@url' => Link::fromTextAndUrl($this->t('here'), $url)->toString(),
      ]),
      '#required' => TRUE,
      '#attributes' => [
        'placeholder' => $this->t('E.g. @example', ['@example' => '8.8.8.8']),
      ],
    ];

    $url = Url::fromUri('https://developer.lametric.com/user/devices', $options);

    $form['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#description' => $this->t('Copy from the <b>Api Key</b> table column on the @url page.', [
        '@url' => Link::fromTextAndUrl('Devices', $url)->toString(),
      ]),
      '#required' => TRUE,
    ];

    $url = Url::fromUri('https://developer.lametric.com/icons', $options);

    $form['icon'] = [
      '#type' => 'number',
      '#title' => $this->t('Icon ID'),
      '#description' => $this->t('Select an icon on the @url page.', [
        '@url' => Link::fromTextAndUrl('Gallery', $url)->toString(),
      ]),
      '#attributes' => [
        'placeholder' => $this->t('E.g. @example', ['@example' => 2867]),
      ],
    ];

    $form['text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Text'),
      '#required' => TRUE,
      '#attributes' => [
        'placeholder' => $this->t('E.g. @example', [
          '@example' => $this->t('Hello!'),
        ]),
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $response = $this->notification->send(
      $form_state->getValue('ip'),
      $form_state->getValue('key'),
      $form_state->getValue('text'),
      $form_state->getValue('icon')
    );

    if ($response instanceof \Exception) {
      $this->messenger()->addError($response->getMessage());
    }
    elseif (is_numeric($response)) {
      $this->messenger()->addStatus($this->t('The notification has been successfully sent.'));
    }
    else {
      $this->messenger()->addWarning($response);
    }
  }

}
